<?php
namespace Teknisa\FAI\Modules\Tabelas\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Setforms2\Shared\ORM\DateTime;

class Fai00600ClienteServico extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $environment;
    protected $crudService;
    protected $parameterBag;
    protected $dataSourceName;
    protected $parameterBagColumns;
    protected $entityManager;
    protected $connection;

    public function __construct($crudService, $parameterBag, $environment, $entityManager) {

        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->entityManager  = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    private function addServClie($row) {
        $date = DateTime::createFromFormat('d/m/Y', $row["DTSERVFATU"]);
        $servclie = new \Teknisa\Model\Planejamento\Entities\Servclie();
        $servclie->setNrorg($this->environment->getNrOrg());
        $servclie->setCdfilial($row["CDFILIAL"]);
        $servclie->setCdservfili($row["CDSERVFILI"]);
        $servclie->setCdcliente($row["CDCLIENTE"]);
        $servclie->setDtservfatu($date);
        $servclie->setQtefetmini($row["QTEFETMINI"]);
        $servclie->setVrservfatu($row["VRSERVFATU"]);
        $servclie->setQtvolproservcli($row["QTVOLPROSERVCLI"]);
        $servclie->setDsemissercli($row["DSEMISSERCLI"]);
        $servclie->setIdativatabcor($row["IDATIVATABCOR"]);
        $servclie->setIdconsterctabcor($row["IDCONSTERCTABCOR"]);
        $servclie->setDtinclusao(new \DateTime());
        $this->entityManager->persist($servclie);
        $this->entityManager->flush();
    }

    private function addHistServFatu($row) {
        $dateAlt = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTALTEFATHIS']);
        $dateServ = DateTime::createFromFormat('d/m/Y', $row['DTSERVFATU']);
        $histservfatu = new \Teknisa\Model\Faturamento\Entities\Histservfatu();
        $histservfatu->setNrorg($this->environment->getNrOrg());
        $histservfatu->setCdfilial($row['CDFILIAL']);
        $histservfatu->setCdservfili($row['CDSERVFILI']);
        $histservfatu->setCdcliente($row['CDCLIENTE']);
        $histservfatu->setVrservfathis($row['VRSERVFATHIS']);
        $histservfatu->setDtaltefathis($dateAlt);
        $histservfatu->setDtsrvfatuhis($dateServ);
        $histservfatu->setDtinclusao(new \DateTime());
        $histservfatu->setDtultatu(new \DateTime());
        $histservfatu->setCdoperinclusao($this->environment->getCdOperador());
        $this->entityManager->persist($histservfatu);
        $this->entityManager->flush();
        
    }

    public function adicionarRelacionamento(DTO\Request\Row $request, DTO\Response $response) {

        $rows = $request->getRow();
        $clientesJaAdicionados = array();
        $cdClientesJaAdicionados = array();

        foreach ($rows as $row) {

            $params = array(
                'nrorg' => $this->environment->getNrOrg(),
                'cdservfili' => $row['CDSERVFILI'],
                'cdcliente' => $row['CDCLIENTE'],
                'cdfilial' => $row['CDFILIAL']
            );

            if($row['__data_source_name'] === 'servclie') {

                $servclie = $this->entityManager->getRepository('\Teknisa\Model\Planejamento\Entities\Servclie')->findOneBy($params);

                if(empty($servclie)) {
                    $this->addServClie($row);
                } else {
                    $clientesJaAdicionados[] = array("NMRAZSOCCLIE" => $row["NMRAZSOCCLIE"], "NMSERVICO" => $row["NMSERVICO"]);
                    $cdClientesJaAdicionados[] = $row["CDCLIENTE"];
                }

            } else if($row['__data_source_name'] === 'histservfatu' && !in_array($row['CDCLIENTE'], $cdClientesJaAdicionados)) {
                
                $histservfatu = $this->entityManager->getRepository('\Teknisa\Model\Faturamento\Entities\Histservfatu')->findOneBy($params);
                
                if(empty($histservfatu) || !is_null($histservfatu)) {
                    
                    $this->addHistServFatu($row);
                }
            }
        }

        if (count($clientesJaAdicionados) > 0) {
            $response->addDataSet(new DataSet("clientesJaAdicionados", $clientesJaAdicionados));
        }
        
    }
    public function saveCorrecao(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $sql = "INSERT INTO CORRPRECSERV (
                    CDFILIAL, 
                    CDSERVFILI, 
                    CDCLIENTE, 
                    VRPEINICSERV, 
                    VRPEFIMSERV, 
                    VRPECORRSERV, 
                    VRNOVOPRECOSERV,
                    QTINISERV, 
                    QTFINSERV, 
                    NRORG, 
                    DTINCLUSAO, 
                    DTULTATU, 
                    NRORGINCLUSAO, 
                    CDOPERINCLUSAO, 
                    NRORGULTATU, 
                    CDOPERULTATU, 
                    IDATIVO) VALUES (
                        :CDFILIAL,
                        :CDSERVFILI,
                        :CDCLIENTE,
                        :VRPEINICSERV,
                        :VRPEFIMSERV,
                        :VRPECORRSERV,
                        :VRNOVOPRECOSERV,
                        :QTINISERV,
                        :QTFINSERV,
                        :NRORG,
                        :DTINCLUSAO,
                        :DTULTATU,
                        :NRORGINCLUSAO,
                        :CDOPERINCLUSAO,
                        :NRORGULTATU,
                        :CDOPERULTATU,
                        :IDATIVO
                    )";
        $currentDate = $this->environment->getCurrentTime()->format('d/m/Y h:m:s');
        $params = array(
            'CDFILIAL'        => $row['CDFILIAL'],
            'CDSERVFILI'      => $row['CDSERVFILI'],
            'CDCLIENTE'       => $row['CDCLIENTE'],
            'VRPEINICSERV'    => $row['VRPEINICSERV'],
            'VRPEFIMSERV'     => $row['VRPEFIMSERV'],
            'VRPECORRSERV'    => $row['VRPECORRSERV'],
            'VRNOVOPRECOSERV' => $row['VRNOVOPRECOSERV'],
            'QTINISERV'       => $row['QTINISERV'],
            'QTFINSERV'       => $row['QTFINSERV'],
            'NRORG'           => $this->environment->getNrOrg(),
            'DTINCLUSAO'      => $currentDate,
            'DTULTATU'        => $currentDate,
            'NRORGINCLUSAO'   => $this->environment->getNrOrg(),
            'CDOPERINCLUSAO'  => $this->environment->getCdOperador(),
            'NRORGULTATU'     => $this->environment->getNrOrg(),
            'CDOPERULTATU'    => $this->environment->getCdOperador(),
            'IDATIVO'         => 'S'
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function removeCorrecao(DTO\Request\DataSet $request, DTO\Response $response){
        $rows = $request->getDataSet()->getRows();
        $sql = "DELETE FROM CORRPRECSERV 
                    WHERE
                        CDFILIAL     = :CDFILIAL     AND
                        CDSERVFILI   = :CDSERVFILI   AND
                        CDCLIENTE    = :CDCLIENTE    AND
                        VRPEINICSERV = :VRPEINICSERV AND
                        NRORG = :NRORG";
        $std = $this->connection->prepare($sql);
        foreach($rows as $row){
            $params = array(
                'CDFILIAL'     => $row['CDFILIAL'],
                'CDSERVFILI'   => $row['CDSERVFILI'],
                'CDCLIENTE'    => $row['CDCLIENTE'],
                'VRPEINICSERV' => $row['VRPEINICSERV'],
                'NRORG'        => $row['NRORG']
            );
            $std->execute($params);
        }
    }
    
    public function editaCorrecao(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        foreach($rows as $row){
            $sql = "UPDATE CORRPRECSERV
                    SET VRPEINICSERV = :P_NEWVRPEINICSERV,
                        QTFINSERV = :P_QTFINSERV,
                        QTINISERV = :P_QTINISERV,
                        VRPECORRSERV = :P_VRPECORRSERV,
                        VRPEFIMSERV = :P_VRPEFIMSERV,
                        VRNOVOPRECOSERV=:P_VRNOVOPRECOSERV
                    WHERE
                        CDFILIAL = :P_CDFILIAL AND
                        CDSERVFILI = :P_CDSERVFILI AND
                        CDCLIENTE = :P_CDCLIENTE AND
                        VRPEINICSERV = :P_VRPEINICSERV AND
                        NRORG = :P_NRORG";
            
            $params = array(
                "P_CDFILIAL" => $row['CDFILIAL'],    
                "P_CDSERVFILI" => $row['CDSERVFILI'],    
                "P_CDCLIENTE" => $row['CDCLIENTE'],
                "P_VRPEINICSERV" => $row['VRPEINICSERV_BKP'],            
                "P_NRORG" => $row['NRORG'],
                "P_NEWVRPEINICSERV" => $row['NEWVRPEINICSERV'],
                "P_VRNOVOPRECOSERV"=>$row['VRNOVOPRECOSERV'],
                "P_QTFINSERV" => $row['QTFINSERV'],
                "P_QTINISERV" => $row['QTINISERV'],
                "P_VRPECORRSERV" => $row['VRPECORRSERV'],
                "P_VRPEFIMSERV" => $row['VRPEFIMSERV']
            );
            try {
                $this->connection->beginTransaction();
                $std = $this->connection->prepare($sql);
                $std->execute($params);
                $this->connection->commit();
            } catch (Exception $e) {
                $this->connection->rollback();
                throw $e;
            }
        }
    }
    
    public function deletaHistorico(DTO\Request\Row $request, DTO\Response $response){
        
        $row = $request->getRow();
        
        $sql = "DELETE FROM HISTSERVFATU
                WHERE
                  CDFILIAL = :P_CDFILIAL AND
                  CDSERVFILI = :P_CDSERVFILI AND
                  CDCLIENTE = :P_CDCLIENTE";
                  
        
        $params = array(
            "P_CDFILIAL" => $row['CDFILIAL'],    
            "P_CDSERVFILI" => $row['CDSERVFILI'],    
            "P_CDCLIENTE" => $row['CDCLIENTE']
        );
        
        try {
            $std = $this-> connection->prepare($sql);
            $std->execute($params);
            
        } catch (Exception $e) {
            throw $e;
        }
    }
    
    public function removeClientServ (DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->connection->beginTransaction();
        $sqlDeleteHist = "DELETE FROM HISTSERVFATU
                            WHERE
                              CDFILIAL = :CDFILIAL AND
                              CDSERVFILI = :CDSERVFILI AND
                              CDCLIENTE = :CDCLIENTE";
                              
        $sqlDeleteClientServ = "DELETE FROM SERVCLIE
                                    WHERE
                                      CDFILIAL = :CDFILIAL AND
                                      CDSERVFILI = :CDSERVFILI AND
                                      CDCLIENTE = :CDCLIENTE";
                  
        
        $params = array(
            "CDFILIAL"   => $row['CDFILIAL'],    
            "CDSERVFILI" => $row['CDSERVFILI'],    
            "CDCLIENTE"  => $row['CDCLIENTE']
        );
        
        try {
            $std = $this->connection->prepare($sqlDeleteHist);
            $std->execute($params);
            $std = $this->connection->prepare($sqlDeleteClientServ);
            $std->execute($params);
            $this->connection->commit();
        } catch (Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    private function verifyInterval($rows){
        foreach($rows as $row){
            if(isset($row['verifyInterval']) && $row['verifyInterval']){
                $conditions = array(
                    array(
                        "columnName" => 'CDFILIAL' ,
                        "operator"   => '=',
                        "value"      => $row['CDFILIAL']
                    ),
    			    array(
                        "columnName" => 'CDCLIENTE' ,
                        "operator"   => '=',
                        "value"      => $row['CDCLIENTE']
                    ),
    			    array(
                        "columnName" => 'CDSERVFILI' ,
                        "operator"   => '=',
                        "value"      => $row['CDSERVFILI']
                    ),
    			    array(
                        "columnName" => 'VRPEINICSERV' ,
                        "operator"   => '>',
                        "value"      => $row['VRPEINICSERV']
                    ),
    			    array(
                        "columnName" => 'VRPEFIMSERV' ,
                        "operator"   => '<',
                        "value"      => $row['VRPEFIMSERV']
                    ),
    			    array(
                        "columnName" => 'OLDVRPEINICSERV' ,
                        "operator"   => '<',
                        "value"      => $row['VRPEINICSERV_BKP']
                    )
                );
                $dataSet = $this->crudService->getFilteredDataSet("overlap_old_correcao", $conditions);
    			$exists = $dataSet->getRows();
    			if($exists[0]['QTDE'] > 0){
    			    throw new \Exception('Operação bloqueada!<br>Os valores sobrepõe os registros já existentes.');
    			}
            }
        }
    }
    
    public function verificaDataVigencia(DTO\Request\Row $request, DTO\Response $response){
        $query = 'SELECT MAX(DTSRVFATUHIS) DTSRVFATUHIS
                      FROM HISTSERVFATU
                     WHERE CDCLIENTE  = :CDCLIENTE 
                       AND CDFILIAL   = :CDFILIAL
                       AND CDSERVFILI = :CDSERVFILI';
        $row = $request->getRow();
        $params = array(
            'CDCLIENTE'  => $row['CDCLIENTE'],
            'CDFILIAL'   => $row['CDFILIAL'],
            'CDSERVFILI' => $row['CDSERVFILI']
        );
        $std = $this->connection->prepare($query);
        $std->execute($params);
        $data = explode(' ',$std->fetch()['DTSRVFATUHIS'])[0];
        $dataFormat = DateTime::createFromFormat('d/m/Y', $data);
        $dataSelecionada = DateTime::createFromFormat('d/m/Y', $row['DTSRVFATUHIS']);
        $result = $dataSelecionada < $dataFormat;
        $response->addDataSet(new DataSet('result', array($result)));
    }
    
    public function searchDataServClie(DTO\Request\Row $request, DTO\Response $response) {
        $query = 'SELECT S.CDCLIENTE, S.CDFILIAL, S.CDSERVFILI, NVL(H.DTSRVFATUHIS, S.DTSERVFATU) AS DTSERVFATU, NVL(H.VRSERVFATHIS, S.VRSERVFATU) AS VRSERVFATU, S.VRSERVFATDOM, S.VRSERVFATSEG, S.VRSERVFATTER, S.VRSERVFATQUA, S.VRSERVFATQUI, S.VRSERVFATSEX, S.VRSERVFATSAB, S.VRSERVFATFER, S.QTEFETMINI, S.QTEFETMINDOM, S.QTEFETMINSEG, S.QTEFETMINTER, S.QTEFETMINQUA, S.QTEFETMINQUI, S.QTEFETMINSEX, S.QTEFETMINSAB, S.QTEFETMINFER, S.IDEXVLRVENDA, S.IDVRPRFIXAUT, S.DTULTATUSRVC, S.DSEMISSERCLI, S.DSPRODEMISDOM, S.DSPRODEMISSEG, S.DSPRODEMISTER, S.DSPRODEMISQUA, S.DSPRODEMISQUI, S.DSPRODEMISSEX, S.DSPRODEMISSAB, S.DSPRODEMISFER, S.CDPTOVNDFATUR, S.IDUTNFPREIMP, S.CDGRUPOFISC, S.CDSERVFATU, S.QTVOLPROSERVCLI, S.IDATIVATABCOR, S.NRORG, S.DTINCLUSAO, S.DTULTATU, S.NRORGINCLUSAO, S.CDOPERINCLUSAO, S.NRORGULTATU, S.CDOPERULTATU, S.IDATIVO, S.IDCONSTERCTABCOR, S.IDCONSTERCCOMP FROM SERVCLIE       S LEFT JOIN HISTSERVFATU   H ON H.CDCLIENTE = S.CDCLIENTE AND H.CDFILIAL = S.CDFILIAL AND H.CDSERVFILI = S.CDSERVFILI AND H.DTSRVFATUHIS = ( SELECT MAX(DTSRVFATUHIS) FROM HISTSERVFATU H2 WHERE H2.CDCLIENTE = :CDCLIENTE AND H2.CDFILIAL = :CDFILIAL AND H2.CDSERVFILI = :CDSERVFILI AND H2.DTSRVFATUHIS <= :P_SYSDATE ) WHERE S.CDCLIENTE = :CDCLIENTE AND S.CDFILIAL = :CDFILIAL AND S.CDSERVFILI = :CDSERVFILI';
        $row = $request->getRow();
        $params = array(
            'CDCLIENTE' => $row['CDCLIENTE'],
            'CDFILIAL'   => $row['CDFILIAL'],
            'CDSERVFILI' => $row['CDSERVFILI'],
            'P_SYSDATE' => $row['SYSDATE']
        );
        $std = $this->connection->prepare($query);
        $std->execute($params);
        $responseRow = (object) array(
            'DTSERVFATU' => $std->fetch()["DTSERVFATU"],
            'VRSERVFATU' => $std->fetch()["VRSERVFATU"],
            'QTEFETMINI' => $std->fetch()["QTEFETMINI"],
            'DSEMISSERCLI' => $std->fetch()["DSEMISSERCLI"],
            'QTVOLPROSERVCLI' => $std->fetch()["QTVOLPROSERVCLI"],
            'DSMOTALTEHIS' => $std->fetch()["DSMOTALTEHIS"]
        );
        $response->addDataSet(new DataSet('result', array($responseRow)));
    }

}